@@ -1,16 +1,13 @@
 package App::RPi::EnvUI;
 
 use App::RPi::EnvUI::API;
-use App::RPi::EnvUI::Auth;
-use Data::Dumper;
 use Dancer2;
 use Dancer2::Plugin::Auth::Extensible;
 use Dancer2::Core::Request;
-#use Dancer2::Session::JSON;
 use Mock::Sub no_warnings => 1;
 use POSIX qw(tzset);
 
-our $VERSION = '0.26';
+our $VERSION = '0.27';
 
 my $db = App::RPi::EnvUI::DB->new;
 my $api = App::RPi::EnvUI::API->new(db => $db);
@@ -22,6 +19,10 @@ my $log = $api->log()->child('webapp');
 
 $api->_config_light();
 
+#FIXME: add set/get subs
+
+$api->set_light_times;
+
 #
 # fetch routes
 #
@@ -31,14 +32,7 @@ get '/' => sub {
         $log->_7("entered");
 
         # return template 'main';
-        return template 'test', {requester => request->address};
-        # return template 'switch';
-        # return template 'switch2';
-        # return template 'menu';
-        # return template 'drag';
-        # return template 'flip';
-        # return template 'graph';
-        # return template 'graph_live';
+        return template 'test';
     };
 
 # fetch graph code
@@ -61,18 +55,22 @@ post '/login' => sub {
     };
 
 any '/logout' => sub {
+#        cookie session expires => '-1 day';
         app->destroy_session;
+        redirect '/';
     };
 
 get '/logged_in' => sub {
-        if (session 'logged_in_user'){
+        if (session 'logged_in_user' || request->address eq '127.0.0.1'){
             return to_json { status => 1 };
         }
         return to_json {status => 0};
     };
 
 get '/time' => sub {
-        return join ':', (localtime)[2, 1];
+        my ($h, $m) = (localtime)[2, 1];
+        $m = "0$m" if length $m < 2;
+        return join ':', ($h, $m);
     };
 
 get '/stats' => sub {
@@ -84,11 +82,6 @@ get '/light' => sub {
         $log->_7("entered");
         return to_json $api->_config_light();
     };
-get '/water' => sub {
-        my $log = $log->child('/water');
-        $log->_7("entered");
-        return to_json $api->_config_water();
-    };
 get '/get_config/:want' => sub {
         my $want = params->{want};
 
@@ -125,8 +118,9 @@ get '/fetch_env' => sub {
         my $log = $log->child('/fetch_env');
 
         my $data = $api->env();
-
+        
         $log->_5("temp: $data->{temp}, humidity: $data->{humidity}");
+
         return to_json {
             temp => $data->{temp},
             humidity => $data->{humidity}
@@ -137,9 +131,11 @@ get '/fetch_env' => sub {
 # set routes
 #
 
-get '/set_aux/:aux/:state' => sub {
-
-        if ((request->address ne '127.0.0.1' && ! session 'logged_in_user') || $ENV{UNIT_TEST}){
+get '/set_aux_state/:aux/:state' => sub {
+        
+        if (
+            (request->address ne '127.0.0.1' && ! session 'logged_in_user')
+            || $ENV{UNIT_TEST}){
             return to_json {
                     error => 'unauthorized request. You must be logged in'
             };
@@ -148,19 +144,13 @@ get '/set_aux/:aux/:state' => sub {
         my $aux_id = params->{aux};
         my $state = $api->_bool(params->{state});
 
-        my $log = $log->child('/fetch_env');
+        my $log = $log->child('/set_env');
         $log->_5("aux_id: $aux_id, state: $state");
 
         $state = $api->aux_state($aux_id, $state);
 
         $log->_6("$aux_id updated state: $state");
 
-        my $override = $api->aux_override($aux_id) ? 0 : 1;
-        $log->_6("$aux_id override: $override");
-
-        $override = $api->aux_override($aux_id, $override);
-        $log->_6("$aux_id new override: $override");
-
         $api->switch($aux_id);
 
         return to_json {
@@ -169,6 +159,32 @@ get '/set_aux/:aux/:state' => sub {
         };
     };
 
+get '/set_aux_override/:aux/:override' => sub {
+
+        if (
+            (request->address ne '127.0.0.1' && ! session 'logged_in_user')
+            || $ENV{UNIT_TEST}){
+            return to_json {
+                    error => 'unauthorized request. You must be logged in'
+            };
+        }
+        
+        my $aux_id = params->{aux};
+        my $override = $api->_bool(params->{override});
+
+        my $log = $log->child('/set_aux_override');
+        $log->_5("aux_id: $aux_id, override $override");
+
+        $override = $api->aux_override($aux_id, $override);
+
+        $log->_6("$aux_id updated override $override");
+
+        return to_json {
+            aux => $aux_id,
+            override => $override,
+        };
+
+    };
 
 true;
 
@@ -194,41 +210,26 @@ Now direct your browser at your Pi, on port 3000:
 
 =head1 DESCRIPTION
 
-A self-contained web application that runs on a Raspberry Pi and monitors and
-manages an indoor grow room environment, with an API that can be used external
-to the web app itself.
+*** This is trial software until v1.00 is released. It's still a constant work
+in progress, so the docs are awful, but I am trying to improve them as I learn
+the things I need to know to get where I want to be.***
 
-***NOTE*** This distribution is still in heavy development. It will unit test
-on any *nix PC, but at this time, it will only run correctly on a Raspberry Pi
-with L<wiringPi|http://wiringpi.com> installed. We also require C<sudo> to run
-the webapp, due to limitations in other software I rely upon, but I've got fixes
-in the works to eliminate the C<sudo> requirement.
+*** Note that my focus hasn't been one about detailed security, so please, if
+you desire to test this software, ensure it is not Internet facing, and you have
+adequate protection from undesired access ***
+
+A self-contained, one-page web application that runs on a Raspberry Pi and
+monitors and manages an indoor grow room environment, with an API that can be
+used external to the web app itself.
 
 This distribution reads environmental sensors, turns on/off devices based on
-specific thresholds, contains an adjustable grow light timer, as well as
-feeding timers.
+specific thresholds, contains an adjustable grow light timer, and five extra
+auxillary channels that you can configure for your own use.
 
 The software connects to Raspberry Pi GPIO pins for each C<"auxillary">, and at
 specific times or thresholds, turns on and or off the 120/240v devices that
 you've relayed to that voltage (if you choose to use this functionality).
 
-Whether or not you connect/use the automation functionality, the web UI is a
-one-page app that relies on jQuery/Javascript to pull updates from the server,
-push changes to the server, and display up-to-date live information relating to
-all functionality.
-
-Buttons are present to manually override devices (turn on/off) outside of their
-schedule or whether they've hit thresholds or not. Devices that have been
-overridden through the web UI will not be triggered by automation until the
-override is lifted.
-
-The current temperature and humidity is prominently displayed as are all of the
-other features/statistics.
-
-This is pretty much a singleton application, meaning that all web browsers open
-to the app's UI page will render updates at the same time, regardless if another
-browser or the automation makes any changes.
-
 =head1 WHAT IT DOES
 
 Reads temperature and humidity data via a hygrometer sensor through the
@@ -253,15 +254,50 @@ We also include a grow light scheduler, so that you can connect your light, set
 the schedule, and we'll manage it. The light has an override switch in the UI,
 but that can be disabled to prevent any accidents.
 
-...manages auto-feeding too, but that's not any where near complete yet.
-
 =head1 HOW IT WORKS
 
 Upon installation of this module, a new directory C<envui> will be created in
 your home directory. All of the necessary pieces of code required for this web
 app to run are copied into that directory. You simply change into that
 directory, and run C<sudo plackup bin/app.pl>, then point your browser to
-C<http://raspberry.pi:3000>.
+C<http://raspberry.pi:3000>. I haven't integrated it into a full-blown web
+server as of yet.
+
+There are eight auxillary channels (UI buttons that connect to GPIO pins to turn
+devices on or off). Three are dedicated to specific functionality. The first
+(aux1) is used for temperature sensor duties. The second, humidity sensor
+duties. The third is set up to manage a single grow lamp. The remaining five
+auxillaries can be set and connected to whatever you please, but these channels
+do not have any logic behind them yet; they're just on or off.
+
+=head1 WEB UI
+
+The UI and infrastructure behind it is in its infancy. There are vast changes
+that I'll be making. Currently I have:
+
+    - a reasonably nice menu system, with the current time displayed
+    - all auxillaries are movable objects within the page
+    - if objects are moved, the layout will be preserved across a refresh
+    - ability to easily reset page layout to default
+    - temp, humidity and light auxillary objects will be in override state if
+      the state is changed in the UI (ie. automation routines will skip them)
+    - the memory footprint of a long run is very manageable (7-15MB)
+    - longest unchanged runtime: 178 hours
+    - it's a singleton; all browsers pointed to the UI will see the same state,
+      with updates every three seconds maximum
+    - design is geared to a 5" LCD touchscreen for attaching to the device
+      itself, but renders reasonably well on any device size
+    - authentication is required for any routes that set state of any kind
+    - everything is stored in a DB backend
+
+=head1 HOWTO
+
+I'm not going into detail here yet. Look at the
+L<App::RPi::EnvUI::Configuration> documentation to get an idea of the config
+file.
+
+Map the C<pin> of each aux in the configuration file to a GPIO pin. Start up the
+app per L</HOW IT WORKS>. Go to the webpage in an HTML5-capable browser.
 
 =head1 ROUTES
 
@@ -283,15 +319,6 @@ the page.
 
 Return: JSON
 
-=head2 /water
-
-Use: Internal
-
-Returns a JSON string containing the configuration for the C<water> (feeding)
-section of the page.
-
-Return: JSON
-
 =head2 /get_config/:want
 
 Use: Internal
@@ -346,7 +373,7 @@ humidity). Takes no parameters.
 
 Return: JSON. A JSON string in the form C<{"temp": "Int", "humidity": "Int"}>
 
-=head2 /set_aux/:aux/:state
+=head2 /set_aux_state/:aux/:state
 
 Use: Internal
 
@@ -367,6 +394,31 @@ Mandatory: Bool. The state of the auxillary after the button change.
 Return: JSON. Returns the current state of the auxillary in the format
 C<>{"aux": "aux_name", "state": "bool"}>.
 
+Note: The UI user must be logged in to access this route.
+
+=head2 /set_aux_override/:aux/:state
+
+Use: Internal
+
+Sets the override status of an auxillary channel, when an on-change event occurs
+to a button that is associated with an auxillary.
+
+Parameters:
+
+    :aux
+
+Mandatory: String. The string name of the auxillary channel to change state on
+(eg: C<aux1>).
+
+    :state
+
+Mandatory: Bool. The override status of the auxillary after the button change.
+
+Return: JSON. Returns the current state of the auxillary in the format
+C<>{"aux": "aux_name", "override": "bool"}>.
+
+Note: The UI user must be logged in to access this route.
+
 =head1 AUTHOR
 
 Steve Bertrand, E<lt>steveb@cpan.org<gt>
