@@ -4,14 +4,13 @@ use App::RPi::EnvUI::DB;
 use App::RPi::EnvUI::Event;
 use Carp qw(confess);
 use Crypt::SaltedHash;
-use Data::Dumper;
 use DateTime;
 use JSON::XS;
 use Logging::Simple;
 use Mock::Sub no_warnings => 1;
 use RPi::WiringPi::Constant qw(:all);
 
-our $VERSION = '0.26';
+our $VERSION = '0.27';
 
 # mocked sub handles for when we're in testing mode
 
@@ -94,56 +93,37 @@ sub action_temp {
     }
 }
 sub action_light {
-    my ($self, $dt) = @_;
+    my ($self, %test_conf) = @_;
 
     my $log = $log->child('action_light');
-    
-    my $now;
 
-    if (defined $dt){
-        $now = $dt;
-    }
-    else {
-        $now = DateTime->now(time_zone => $self->_config_core('time_zone'));
-    }
+    my $aux      = $self->_config_control('light_aux');
+    my $pin      = $self->aux_pin($aux);
+    my $override = $self->aux_override($aux);
 
-    if ($self->_config_light('on_hours') == 0){
-        $self->aux_state($self->_config_control('light_aux'), OFF);
-        write_pin($self->aux_pin($self->_config_control('light_aux')), LOW);
-        return;
-    }
-    if ($self->_config_light('on_hours') == 24){
-        if(! $self->aux_state($self->_config_control('light_aux'))){
-            $self->db()->update('light', 'value', time(), 'id', 'on_since');
-            $self->aux_state($self->_config_control('light_aux'), ON);
-            pin_mode($self->_config_control('light_aux'),  OUTPUT);
-            write_pin($self->aux_pin($self->_config_control('light_aux')), HIGH);
+    return if $override;
+
+    my $on_time  = $self->_config_light('on_time');
+    my $off_time = $self->_config_light('off_time');
+
+    my $on_hours = defined $test_conf{on_hours}
+        ? $test_conf{on_hours}
+        : $self->_config_light('on_hours');
+
+    my $now = defined $test_conf{now} ? $test_conf{now} : time;
+
+    if (($on_hours == 24) || ($now > $on_time && $now < $off_time)){
+        if (! $self->aux_state($aux)){
+            $self->aux_state($aux, ON);
+            pin_mode($pin, OUTPUT);
+            write_pin($pin, HIGH);
         }
-        return;
     }
-
-    my $on_since  = $self->_config_light('on_since');
-    my $light_on  = $self->light_on;
-
-    # print "now: " . $now->ymd . " " . $now->hms . "\n";
-    # print "on: " . $light_on->ymd . " " . $light_on->hms . "\n";
-
-    if (! $on_since  && $now > $light_on){
-        $self->db()->update('light', 'value', time(), 'id', 'on_since');
-        $self->aux_state($self->_config_control('light_aux'), ON);
-        pin_mode($self->_config_control('light_aux'),  OUTPUT);
-        write_pin($self->aux_pin($self->_config_control('light_aux')), HIGH);
-    }
-
-    if ($on_since) {
-        my $light_off = $self->light_off;
-        # print "off: " . $light_off->ymd . " " . $light_off->hms . "\n";
-        if ($now > $light_off) {
-            $self->db()->update('light', 'value', 0, 'id', 'on_since');
-            $self->aux_state( $self->_config_control('light_aux'), OFF);
-            pin_mode($self->_config_control('light_aux'), OUTPUT);
-            write_pin($self->aux_pin($self->_config_control('light_aux')), LOW);
-        }
+    elsif ($self->aux_state($aux)){
+        $self->aux_state($aux, OFF);
+        pin_mode($pin, OUTPUT);
+        write_pin($pin, LOW);
+        $self->set_light_times;
     }
 }
 sub aux {
@@ -289,14 +269,13 @@ sub graph_data {
                 $count++;
             }
         }
+        
+        next if $_->[2] > 120;
+        last if $count == 300;
+        push @{ $data{temp} }, [ $count, $_->[2] ];
+        push @{ $data{humidity} }, [ $count, $_->[3] ];
 
-
-        # every 4 entries; typically we have 4 polls per minute
-        if ($check % 4 == 1) {
-            push @{ $data{temp} }, [ $count, $_->[2] ];
-            push @{ $data{humidity} }, [ $count, $_->[3] ];
-            $count++;
-        }
+        $count++;
         $check++;
     }
 
@@ -424,39 +403,29 @@ sub env_temp_aux {
     my $self = shift;
     return $self->_config_control('temp_aux');
 }
-sub light_on {
+sub set_light_times {
     my ($self) = @_;
 
-    my ($on_hour, $on_min) = split /:/, $self->_config_light('on_at');
+    my $on_at = $self->_config_light('on_at');
 
-    my $now = DateTime->now(
-        time_zone => $self->db()->config_core('time_zone')
-    );
+    my $time = time;
+    $time += 30 until localtime($time) =~ /$on_at:/;
 
-    my $on
-      = $now->clone()->set_hour($on_hour)->set_minute($on_min)->set_second(0);
+    my $hrs = $self->_config_light('on_hours');
 
-    return $on;
-}
-sub light_off {
-    my ($self, $dt) = @_;
+    my $on_time = $time;
+    my $off_time = $on_time + $hrs * 3600;
 
-    my $on_hours = $self->_config_light('on_hours');
-    my $on_since = $self->_config_light('on_since');
+    my $now = time;
 
-    if (defined $dt){
-        return $dt->clone()->add(hours => $on_hours);
-    }
-    else {
-        $dt = DateTime->from_epoch(
-            time_zone => $self->_config_core('time_zone'),
-            epoch => $on_since
-        );
-
-        $dt->add(hours => $on_hours);
+    if ($now > ($on_time - 86400) && $now < ($off_time - 86400)){
+        $on_time -= 24 * 3600;
+        $off_time -= 24 * 3600;
     }
 
-    return $dt;
+    $self->db()->update('light', 'value', $on_time, 'id', 'on_time');
+    $self->db()->update('light', 'value', $off_time, 'id', 'off_time');
+
 }
 
 # public instance variable methods
@@ -586,24 +555,6 @@ sub _config_light {
 
     return \%conf;
 }
-sub _config_water {
-    my $self = shift;
-    my $want = shift;
-
-    my $water = $self->db()->config_water;
-
-    if (defined $want){
-        return $water->{$want}{value};
-    }
-
-    my %conf;
-
-    for (keys %$water){
-        $conf{$_} = $water->{$_}{value};
-    }
-
-    return \%conf;
-}
 sub _init {
     my ($self) = @_;
 
@@ -720,7 +671,6 @@ sub _parse_config {
     $self->db()->begin;
 
     $self->_reset;
-    $self->_reset_light;
 
     $config = $self->config if ! defined $config;
 
@@ -756,7 +706,7 @@ sub _parse_config {
         $self->db()->update_bulk(@$db_struct, \@data);
     }
 
-    for my $conf_section (qw(control core light water)){
+    for my $conf_section (qw(control core light)){
         my $db_struct = [
             $conf_section,
             'value',
@@ -800,9 +750,6 @@ sub _reset {
 
     #$self->db()->commit;
 }
-sub _reset_light {
-    $_[0]->db()->update('light', 'value', 0, 'id', 'on_since');
-}
 sub _ui_test_mode {
     return -e 't/testing.lck';
 }
@@ -1147,24 +1094,11 @@ Returns as an integer, the current humidity level.
 
 Returns as an integer, the current temperature level.
 
-=head2 light_on
+=head2 set_light_time
 
-Returns a L<DateTime> object with the C<HH:MM> set to the C<on_at> directive in
-the configuration file.
+Sets in the database the values for lights-on and lights-off time.
 
-=head2 light_off($dt)
-
-Returns a L<DateTime> object with the C<on_hours> config directive value added,
-which represents the time the light shall be turned off.
-
-Parameters:
-
-    $dt
-
-Optional, L<DateTime> object. Use only for testing. We'll use this as the on
-time C<DateTime> instead of one received from a call to C<light_on()>.
-
-Returns: C<DateTime> object.
+Takes no parameters, there is no return.
 
 =head2 log
 
@@ -1229,6 +1163,11 @@ Optional, L<RPi::DHT11> object instance.
 
 Return: The sensor object.
 
+=head2 set_light_times
+
+Internal method that sets the light on-off times in the database, once on webapp
+initial startup, then at the end of the lights-off trigger in C<action_light()>.
+
 =head2 switch($aux_id)
 
 Enables/disables the GPIO pin associated with the specified auxillary channel,
